unit AddInDoc;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ActiveX, AxCtrls, AddIn_TLB, StdCtrls, OleCtrls, Menus, MPlayer;

type
  TAddInDoc = class(TActiveForm, IAddInDoc)
    mciWindow: TStaticText;
    mciCtrl: TMediaPlayer;
  private
    { Private declarations }
    FEvents: IAddInDocEvents;
    procedure ActivateEvent(Sender: TObject);
    procedure ClickEvent(Sender: TObject);
    procedure CreateEvent(Sender: TObject);
    procedure DblClickEvent(Sender: TObject);
    procedure DeactivateEvent(Sender: TObject);
    procedure DestroyEvent(Sender: TObject);
    procedure KeyPressEvent(Sender: TObject; var Key: Char);
    procedure PaintEvent(Sender: TObject);
    procedure ResizeEvent(Sender: TObject);
  protected
    { Protected declarations }
    procedure DefinePropertyPages(DefinePropertyPage: TDefinePropertyPage); override;
    procedure EventSinkChanged(const EventSink: IUnknown); override;
    function Get_Active: WordBool; safecall;
    function Get_AutoScroll: WordBool; safecall;
    function Get_AutoSize: WordBool; safecall;
    function Get_AxBorderStyle: TxActiveFormBorderStyle; safecall;
    function Get_BiDiMode: TxBiDiMode; safecall;
    function Get_Caption: WideString; safecall;
    function Get_Color: OLE_COLOR; safecall;
    function Get_Cursor: Smallint; safecall;
    function Get_DoubleBuffered: WordBool; safecall;
    function Get_DropTarget: WordBool; safecall;
    function Get_Enabled: WordBool; safecall;
    function Get_Font: IFontDisp; safecall;
    function Get_HelpFile: WideString; safecall;
    function Get_KeyPreview: WordBool; safecall;
    function Get_PixelsPerInch: Integer; safecall;
    function Get_PrintScale: TxPrintScale; safecall;
    function Get_Scaled: WordBool; safecall;
    function Get_Visible: WordBool; safecall;
    procedure _Set_Font(const Value: IFontDisp); safecall;
    procedure Set_AutoScroll(Value: WordBool); safecall;
    procedure Set_AutoSize(Value: WordBool); safecall;
    procedure Set_AxBorderStyle(Value: TxActiveFormBorderStyle); safecall;
    procedure Set_BiDiMode(Value: TxBiDiMode); safecall;
    procedure Set_Caption(const Value: WideString); safecall;
    procedure Set_Color(Value: OLE_COLOR); safecall;
    procedure Set_Cursor(Value: Smallint); safecall;
    procedure Set_DoubleBuffered(Value: WordBool); safecall;
    procedure Set_DropTarget(Value: WordBool); safecall;
    procedure Set_Enabled(Value: WordBool); safecall;
    procedure Set_Font(var Value: IFontDisp); safecall;
    procedure Set_HelpFile(const Value: WideString); safecall;
    procedure Set_KeyPreview(Value: WordBool); safecall;
    procedure Set_PixelsPerInch(Value: Integer); safecall;
    procedure Set_PrintScale(Value: TxPrintScale); safecall;
    procedure Set_Scaled(Value: WordBool); safecall;
    procedure Set_Visible(Value: WordBool); safecall;
    procedure Play; safecall;
    procedure SetFileName(const FName: WideString); safecall;
  public
    { Public declarations }
    procedure Initialize; override;
  end;

implementation

uses ComObj, ComServ;

{$R *.DFM}

{ TAddInDoc }

procedure TAddInDoc.DefinePropertyPages(DefinePropertyPage: TDefinePropertyPage);
begin
  { Define property pages here.  Property pages are defined by calling
    DefinePropertyPage with the class id of the page.  For example,
      DefinePropertyPage(Class_AddInDocPage); }
end;

procedure TAddInDoc.EventSinkChanged(const EventSink: IUnknown);
begin
  FEvents := EventSink as IAddInDocEvents;
end;

procedure TAddInDoc.Initialize;
begin
  inherited Initialize;
  OnActivate := ActivateEvent;
  OnClick := ClickEvent;
  OnCreate := CreateEvent;
  OnDblClick := DblClickEvent;
  OnDeactivate := DeactivateEvent;
  OnDestroy := DestroyEvent;
  OnKeyPress := KeyPressEvent;
  OnPaint := PaintEvent;
  OnResize := ResizeEvent;

  mciCtrl.Left := 0;
  mciCtrl.Width := Width;
  mciWindow.Left := 0;
  mciWindow.Width := Width;
  mciWindow.Height := Height - mciWindow.Top;
end;

function TAddInDoc.Get_Active: WordBool;
begin
  Result := Active;
end;

function TAddInDoc.Get_AutoScroll: WordBool;
begin
  Result := AutoScroll;
end;

function TAddInDoc.Get_AutoSize: WordBool;
begin
  Result := AutoSize;
end;

function TAddInDoc.Get_AxBorderStyle: TxActiveFormBorderStyle;
begin
  Result := Ord(AxBorderStyle);
end;

function TAddInDoc.Get_BiDiMode: TxBiDiMode;
begin
  Result := Ord(BiDiMode);
end;

function TAddInDoc.Get_Caption: WideString;
begin
  Result := WideString(Caption);
end;

function TAddInDoc.Get_Color: OLE_COLOR;
begin
  Result := OLE_COLOR(Color);
end;

function TAddInDoc.Get_Cursor: Smallint;
begin
  Result := Smallint(Cursor);
end;

function TAddInDoc.Get_DoubleBuffered: WordBool;
begin
  Result := DoubleBuffered;
end;

function TAddInDoc.Get_DropTarget: WordBool;
begin
  Result := DropTarget;
end;

function TAddInDoc.Get_Enabled: WordBool;
begin
  Result := Enabled;
end;

function TAddInDoc.Get_Font: IFontDisp;
begin
  GetOleFont(Font, Result);
end;

function TAddInDoc.Get_HelpFile: WideString;
begin
  Result := WideString(HelpFile);
end;

function TAddInDoc.Get_KeyPreview: WordBool;
begin
  Result := KeyPreview;
end;

function TAddInDoc.Get_PixelsPerInch: Integer;
begin
  Result := PixelsPerInch;
end;

function TAddInDoc.Get_PrintScale: TxPrintScale;
begin
  Result := Ord(PrintScale);
end;

function TAddInDoc.Get_Scaled: WordBool;
begin
  Result := Scaled;
end;

function TAddInDoc.Get_Visible: WordBool;
begin
  Result := Visible;
end;

procedure TAddInDoc._Set_Font(const Value: IFontDisp);
begin
  SetOleFont(Font, Value);
end;

procedure TAddInDoc.Set_AutoScroll(Value: WordBool);
begin
  AutoScroll := Value;
end;

procedure TAddInDoc.Set_AutoSize(Value: WordBool);
begin
  AutoSize := Value;
end;

procedure TAddInDoc.Set_AxBorderStyle(Value: TxActiveFormBorderStyle);
begin
  AxBorderStyle := TActiveFormBorderStyle(Value);
end;

procedure TAddInDoc.Set_BiDiMode(Value: TxBiDiMode);
begin
  BiDiMode := TBiDiMode(Value);
end;

procedure TAddInDoc.Set_Caption(const Value: WideString);
begin
  Caption := TCaption(Value);
end;

procedure TAddInDoc.Set_Color(Value: OLE_COLOR);
begin
  Color := TColor(Value);
end;

procedure TAddInDoc.Set_Cursor(Value: Smallint);
begin
  Cursor := TCursor(Value);
end;

procedure TAddInDoc.Set_DoubleBuffered(Value: WordBool);
begin
  DoubleBuffered := Value;
end;

procedure TAddInDoc.Set_DropTarget(Value: WordBool);
begin
  DropTarget := Value;
end;

procedure TAddInDoc.Set_Enabled(Value: WordBool);
begin
  Enabled := Value;
end;

procedure TAddInDoc.Set_Font(var Value: IFontDisp);
begin
  SetOleFont(Font, Value);
end;

procedure TAddInDoc.Set_HelpFile(const Value: WideString);
begin
  HelpFile := String(Value);
end;

procedure TAddInDoc.Set_KeyPreview(Value: WordBool);
begin
  KeyPreview := Value;
end;

procedure TAddInDoc.Set_PixelsPerInch(Value: Integer);
begin
  PixelsPerInch := Value;
end;

procedure TAddInDoc.Set_PrintScale(Value: TxPrintScale);
begin
  PrintScale := TPrintScale(Value);
end;

procedure TAddInDoc.Set_Scaled(Value: WordBool);
begin
  Scaled := Value;
end;

procedure TAddInDoc.Set_Visible(Value: WordBool);
begin
  Visible := Value;
end;

procedure TAddInDoc.ActivateEvent(Sender: TObject);
begin
  if FEvents <> nil then FEvents.OnActivate;
end;

procedure TAddInDoc.ClickEvent(Sender: TObject);
begin
  if FEvents <> nil then FEvents.OnClick;
end;

procedure TAddInDoc.CreateEvent(Sender: TObject);
begin
  if FEvents <> nil then FEvents.OnCreate;
end;

procedure TAddInDoc.DblClickEvent(Sender: TObject);
begin
  if FEvents <> nil then FEvents.OnDblClick;
end;

procedure TAddInDoc.DeactivateEvent(Sender: TObject);
begin
  if FEvents <> nil then FEvents.OnDeactivate;
end;

procedure TAddInDoc.DestroyEvent(Sender: TObject);
begin
  if FEvents <> nil then FEvents.OnDestroy;
end;

procedure TAddInDoc.KeyPressEvent(Sender: TObject; var Key: Char);
var
  TempKey: Smallint;
begin
  TempKey := Smallint(Key);
  if FEvents <> nil then FEvents.OnKeyPress(TempKey);
  Key := Char(TempKey);
end;

procedure TAddInDoc.PaintEvent(Sender: TObject);
begin
  if FEvents <> nil then FEvents.OnPaint;
  mciCtrl.Refresh;
end;

procedure TAddInDoc.ResizeEvent(Sender: TObject);
begin
  if FEvents <> nil then FEvents.OnResize;
  mciCtrl.Width := Width;
  mciWindow.Width := Width;
  mciWindow.Height := Height - mciWindow.Top;
end;

procedure TAddInDoc.Play;
begin
    mciCtrl.Open;
end;

procedure TAddInDoc.SetFileName(const FName: WideString);
begin
    mciCtrl.FileName := FName;
end;

initialization
  TActiveFormFactory.Create(
    ComServer,
    TActiveFormControl,
    TAddInDoc,
    Class_AddInDoc,
    1,
    '',
    OLEMISC_SIMPLEFRAME or OLEMISC_ACTSLIKELABEL,
    tmApartment);
end.
